// Source file for the GLUT example program



// RING include files

#include "R3Shapes/R3Shapes.h"



// OpenGL include files 

#include <GL/glut.h>



// GLUT variables 

static int GLUTwindow = 0;
static int GLUTwindow_height = 480;
static int GLUTwindow_width = 640;
static int GLUTmouse[2] = { 0, 0 };
static int GLUTbutton[3] = { 0, 0, 0 };
static int GLUTarrows[4] = { 0, 0, 0, 0 };
static int GLUTmodifiers = 0;



// Display variables

static int scaling = 0;
static int translating = 0;
static int rotating = 0;
static float scale = 1.0;
static float center[3] = { 0.0, 0.0, 0.0 };
static float rotation[3] = { 0.0, 0.0, 0.0 };
static float translation[3] = { 0.0, 0.0, -8.0 };
static int show_wireframe = 0;



// Program variables

RNArray<R3Shape *> shapes;



void GLUTRedraw(void)
{
  // Setup viewing transformation
  glLoadIdentity();
  glScalef(scale, scale, scale);
  glTranslatef(translation[0], translation[1], 0.0);

  // Set projection transformation
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0, (GLfloat) GLUTwindow_width /(GLfloat) GLUTwindow_height, 0.1, 100.0);

  // Set camera transformation
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glTranslatef(translation[0], translation[1], translation[2]);
  glScalef(scale, scale, scale);
  glRotatef(rotation[0], 1.0, 0.0, 0.0);
  glRotatef(rotation[1], 0.0, 1.0, 0.0);
  glRotatef(rotation[2], 0.0, 0.0, 1.0);
  glTranslatef(-center[0], -center[1], -center[2]);

  // Clear window 
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // Set lights
  static GLfloat light0_position[] = { 3.0, 4.0, 5.0, 0.0 };
  glLightfv(GL_LIGHT0, GL_POSITION, light0_position);
  static GLfloat light1_position[] = { -3.0, -2.0, -3.0, 0.0 };
  glLightfv(GL_LIGHT1, GL_POSITION, light1_position);

  // Set material
  static GLfloat material1[] = { 0.8, 0.8, 0.8, 1.0 };
  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, material1); 

  // Draw unit cube 
  glColor3f(1.0, 1.0, 1.0);
  R3unit_box.Outline();
  
  // Draw shapes 
  glColor3f(1.0, 0.0, 1.0);
  for (int i = 0; i < shapes.NEntries(); i++) {
    R3Shape *shape = shapes.Kth(i);
    if (show_wireframe) shape->Outline();
    else shape->Draw();
  }
    
  // Swap buffers 
  glutSwapBuffers();
}    



void GLUTStop(void)
{
  // Destroy window 
  glutDestroyWindow(GLUTwindow);

  // Exit
  exit(0);
}



void GLUTResize(int w, int h)
{
  // Resize window
  glViewport(0, 0, w, h);

  // Remember window size 
  GLUTwindow_width = w;
  GLUTwindow_height = h;

  // Redraw
  GLUTRedraw();
}



void GLUTMotion(int x, int y)
{
  // Invert y coordinate
  y = GLUTwindow_height - y;
  
  // Process mouse motion event
  if (rotating) {
    // Rotate model
    rotation[0] += -0.5 * (y - GLUTmouse[1]);
    rotation[2] += 0.5 * (x - GLUTmouse[0]);
  }
  else if (scaling) {
    // Scale window
    scale *= exp(2.0 * (float) (x - GLUTmouse[0]) / (float) GLUTwindow_width);
  }
  else if (translating) {
    // Translate window
    translation[0] += 2.0 * (float) (x - GLUTmouse[0]) / (float) GLUTwindow_width;
    translation[1] += 2.0 * (float) (y - GLUTmouse[1]) / (float) GLUTwindow_height;
  }

  // Remember mouse position 
  GLUTmouse[0] = x;
  GLUTmouse[1] = y;
}



void GLUTMouse(int button, int state, int x, int y)
{
  // Invert y coordinate
  y = GLUTwindow_height - y;
  
  // Process mouse button event
  rotating = (button == GLUT_LEFT_BUTTON);
  translating = (button == GLUT_MIDDLE_BUTTON);
  scaling = (button == GLUT_RIGHT_BUTTON);
  if (rotating || scaling || translating) glutIdleFunc(GLUTRedraw);
  else glutIdleFunc(0);

  // Remember button state 
  int b = (button == GLUT_LEFT_BUTTON) ? 0 : ((button == GLUT_MIDDLE_BUTTON) ? 1 : 2);
  GLUTbutton[b] = (state == GLUT_DOWN) ? 1 : 0;

  // Remember modifiers 
  GLUTmodifiers = glutGetModifiers();

   // Remember mouse position 
  GLUTmouse[0] = x;
  GLUTmouse[1] = y;
}



void GLUTSpecial(int key, int x, int y)
{
  // Invert y coordinate
  y = GLUTwindow_height - y;

  // Process keyboard button event 

  // Remember mouse position 
  GLUTmouse[0] = x;
  GLUTmouse[1] = y;

  // Remember modifiers 
  GLUTmodifiers = glutGetModifiers();

  // Redraw
  GLUTRedraw();
}



void GLUTKeyboard(unsigned char key, int x, int y)
{
  // Flag to determine whether to redraw
  int redraw = 0;

  // Process keyboard button event 
  switch (key) {
  case 'Q':
  case 'q':
    GLUTStop();
    break;
      
  case 'W':
  case 'w':
    show_wireframe = !show_wireframe;
    break;
      
  case 27: // ESCAPE
    GLUTStop();
    break;
  }

  // Redraw 
  if (redraw) GLUTRedraw();

  // Remember mouse position 
  GLUTmouse[0] = x;
  GLUTmouse[1] = GLUTwindow_height - y;

  // Remember modifiers 
  GLUTmodifiers = glutGetModifiers();

  // Redraw
  GLUTRedraw();
}



void GLUTInit(int *argc, char **argv)
{
  // Open window 
  glutInit(argc, argv);
  glutInitWindowPosition(100, 100);
  glutInitWindowSize(GLUTwindow_width, GLUTwindow_height);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH); // | GLUT_STENCIL
  GLUTwindow = glutCreateWindow("OpenGL Viewer");

  // Initialize GLUT callback functions 
  glutReshapeFunc(GLUTResize);
  glutDisplayFunc(GLUTRedraw);
  glutKeyboardFunc(GLUTKeyboard);
  glutSpecialFunc(GLUTSpecial);
  glutMouseFunc(GLUTMouse);
  glutMotionFunc(GLUTMotion);
  glutIdleFunc(0);
    
  // Initialize background color 
  glClearColor(0.0, 0.0, 0.0, 1.0);

  // Initialize lights */
  static GLfloat lmodel_ambient[] = { 0.2, 0.2, 0.2, 1.0 };
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
  static GLfloat light0_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
  glEnable(GL_LIGHT0);
  static GLfloat light1_diffuse[] = { 0.5, 0.5, 0.5, 1.0 };
  glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_diffuse);
  glEnable(GL_LIGHT1);
  glEnable(GL_NORMALIZE);
  glEnable(GL_LIGHTING);

  // Initialize graphics modes 
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_CULL_FACE);
}



void GLUTMainLoop(void)
{
  // Run main loop -- never returns 
  glutMainLoop();
}



static int CreateShapes(void)
{
  // Create a triangle 
  R3TriangleVertex *tp1 = new R3TriangleVertex(R3Point(0.0, 0.0, 0.0));
  R3TriangleVertex *tp2 = new R3TriangleVertex(R3Point(1.0, 0.0, 0.0));
  R3TriangleVertex *tp3 = new R3TriangleVertex(R3Point(1.0, 1.0, 0.0));
  R3Triangle *t1 = new R3Triangle(tp1, tp2, tp3);
  shapes.Insert(t1);

  // Create a circle
  R3Circle *c1 = new R3Circle(R3zero_point + R3posz_vector * 0.25, 0.25, R3posy_vector);

  // Create a box
  R3Box *box1 = new R3Box(0.25, -0.75, 0.25, 0.75, -0.25, 0.75);
  shapes.Insert(box1);

  // Create a cylinder
  R3Cylinder *cyl1 = new R3Cylinder(R3Point(0.66, -0.66, -0.66), R3Point(0.33, -0.33, -0.33), 0.25);
  shapes.Insert(cyl1);

  // Create a cone
  R3Cone *cone1 = new R3Cone(R3Point(-0.66, 0.66, -0.66), R3Point(-0.33, 0.33, -0.33), 0.25);
  shapes.Insert(cone1);

  // Create a sphere
  R3Sphere *s1 = new R3Sphere(R3Point(-0.5, 0.5, 0.5), 0.25);
  shapes.Insert(s1);

  // Return success
  return 1;
}



int 
main(int argc, char **argv)
{
  // Initialize RING
  if (!R3InitShapes()) exit(1);

  // Initialize GLUT
  GLUTInit(&argc, argv);

  // Create shapes
  if (!CreateShapes()) exit(1);

  // Run GLUT interface
  GLUTMainLoop();

  // Stop GLUT interface
  GLUTStop();

  // Stop RING
  R3StopShapes();

  // Return success 
  return 0;
}











